'''
Created on 26 mars 2018

@author: Tonneau Stephane
'''

import sys
import os
import datetime
import json
import socket
import getpass
import random
import logging

# Utilities


def get_host():
    '''Get the address of the host'''
    host_addr = socket.gethostname()
    if (host_addr is not None) and ("." in host_addr):
        return host_addr
    the_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        # doesn't even have to be reachable
        the_socket.connect(('10.255.255.255', 1))
        host_addr = the_socket.getsockname()[0]
    except Exception:
        host_addr = '127.0.0.1'
    finally:
        the_socket.close()
    return host_addr


def get_user():
    '''Get the logged username'''
    if "USER" in os.environ:
        username = os.environ["USER"]
    elif "LOGNAME" in os.environ:
        username = os.environ["LOGNAME"]
    else:
        username = getpass.getuser()
    return username


def read_config(filename):
    '''Read a configuration file with the adequate configuration parser'''
    configuration = None
    if os.path.isfile(filename):
        if sys.version_info < (3,):
            import ConfigParser
            configuration = ConfigParser.ConfigParser()
        else:
            import configparser
            configuration = configparser.ConfigParser()
    configuration.read(filename)
    return configuration


def get_config(configuration, mode_name, name):
    '''Return a specific definition element from
       the Configuration or the environment'''
    tmp_env = mode_name + "." + name
    if tmp_env in os.environ:
        value = os.environ[tmp_env]
    elif configuration is not None:
        try:
            value = configuration.get(mode_name, name)
        except:
            value = None
    else:
        value = None
    return value


def first_not_null(*args):
    '''Return the first argument that is not None'''
    for value in args:
        if value is not None:
            return value
    return None


def bean_2_dict(value):
    '''Serialize the content of value as a string dictionary'''
    if value is None:
        result = None
    elif isinstance(value, JsonBased):
        result = value.to_dict()
        if not result:
            result = None
    elif isinstance(value, list):
        result = []
        for subval in value:
            result.append(bean_2_dict(subval))
        if not result:
            result = None
    elif isinstance(value, datetime.datetime):
        result = value.strftime("%Y-%m-%dT%H:%M:%SZ")
    else:
        result = value
    return result


class JsonBased(object):
    '''Abstract class that contains serialization methods'''

    def to_json(self):
        '''Serialize the content of the class as a json string'''
        return json.dumps(self.to_dict())

    def to_dict(self):
        '''Serialize the content of the class as a dictionary value,
           ignoring keys starting by _
        '''
        result = {}
        for key in self.__dict__:
            if key[0:1] != "_":
                value = self.__dict__[key]
                tmp_value = bean_2_dict(value)
                if tmp_value is not None:
                    result[key] = tmp_value
        return result


class SystemEnvironment(JsonBased):
    '''Abstract class that encapsulate envirnoment specificities'''

    def __init__(self):
        self.host = get_host()
        self.user = get_user()
        self.configuration = None
        self.job_id = None
        self.job_type = None
        self.main_id = None
        self.environment_type = None

    def get_uid(self):
        """Build a unique id"""
        start_time = datetime.datetime(2018, 3, 1, 0, 0, 0)
        dtime = int(
            (datetime.datetime.utcnow() - start_time).total_seconds()) * 1000000
        uid = str(dtime)

        if self.main_id is None:
            uid = uid + "_" + str(int(random.uniform(0, 999999999999999)))
        else:
            uid = uid + "_" + self.main_id
        uid = uid + "_" + str(int(random.uniform(0, 999)))
        # Build a random unique id
        return uid


class Configuration(JsonBased):
    '''Abstract class that contains relevant System configuration information'''

    def __init__(self, driver_memory=None, executer_memory=None, num_executers=None,
                 queue=None, executor_environment=None):
        # memory used in the context of a spark driver
        self.driver_memory = driver_memory
        # memory used in the context of a spark executor
        self.executer_memory = executer_memory
        # number of executers value
        self.num_executers = num_executers
        # initial number of executers value
        self.init_executors = None
        # minimum number of executers value
        self.min_executors = None
        # maximum number of executers value
        self.max_executors = None
        # Executer env
        self.executor_environment = executor_environment
        # queue value
        self.queue = queue


class LoggingMethod(object):
    '''Abstract class that logs information with a specific approach'''

    def __init__(self, configuration=None):
        self.logging_type = None
        self.configuration = None
        self.logger = logging.getLogger('pidlogger')
        self.logger.setLevel(logging.INFO)
        if not self.logger.handlers:
            hdlr = logging.StreamHandler()
            self.logger.addHandler(hdlr)
        if configuration is not None:
            self.configuration = configuration

    def get_config(self, node_name, name):
        '''Return a specific definition element from
           the Configuration or the environment'''
        return get_config(self.configuration, node_name, name)
