'''
Created on 8 mars 2018

@author: Tonneau Stephane
'''
import datetime
from .base_utils import JsonBased


class Metainformation(JsonBased):
    '''Class that encapsulate some metainformation of pid entries'''

    def __init__(self, uid=None):
        self.uid = uid
        self.timestamp = None


class GeoJSON(JsonBased):
    '''class that contains a geojson polygon definition'''

    def __init__(self, wktDefinition):
        self.type = None
        self.coordinates = None
        if wktDefinition.startswith("POLYGON"):
            tmp_coord = wktDefinition.replace("POLYGON", "").strip()
            tmp_coord = tmp_coord.replace(
                "(", "").replace(")", "").replace(",", "").strip()
            coords = tmp_coord.split(" ")
            point_list = []
            a_point = []
            for num in coords:
                if len(num.strip()) > 0:
                    a_point.append(float(num))
                    if len(a_point) == 2:
                        point_list.append(a_point)
                        a_point = []
            nbr_point = len(point_list)
            if nbr_point == 1:
                self.type = "point"
            elif nbr_point == 2:
                self.type = "envelope"
            else:
                self.type = "polygon"
            self.coordinates = [point_list]


class FileInfo(JsonBased):
    '''Class that encapsulate a file information'''

    INPUT_CONTEXT = "input"
    CREATED_CONTEXT = "created"
    REMOVED_CONTEXT = "removed"

    EOPRODUCT_TYPE = "eoproduct"
    QUICKLOOK_TYPE = "quicklook"
    POSTTEST_TYPE = "posttest"
    VALIDATION_RESULT_TYPE = "validation_result"
    VALIDATION_REPORT_TYPE = "validation_report"
    CONFIG_TYPE = "config"

    def __init__(self, path_val, context_val=CREATED_CONTEXT, type_val=None, geometry=None,
                 checksum=None, size=None, product_id=None, ogc_urn=None):
        '''the context associated with the file
          (see ..CTX constants)
        '''
        self.context = context_val
        '''the type associated with the file
          (see ..TP constants)
        '''
        self.type = type_val
        '''Absolute path to the file or directory'''
        self.path = path_val
        '''Geometry in the GeoJSON format'''
        if geometry is not None:
            self.geometry = GeoJSON(geometry)
        '''Checksum of the file in the format {[SHA|MD5|..]}<checksum value>'''
        self.checksum = checksum
        '''Total size of the file or directory'''
        self.size = size
        '''product_id of the product to which the file or directory refers'''
        self.product_id = product_id
        '''ogc urn reference of the product to which the file or directory refers'''
        self.ogc_urn = ogc_urn


class PidEntry(JsonBased):
    '''Class that can be used to store the pertinent process log information'''

    def __init__(self, product_id, product_type, product_date, job_id=None,
                 proc_type=None, environment=None, start_time=None, user=None, host=None,
                 job_desc=None, job_type=None, parent_job_id=None, end_time=None,
                 exit_code=None, message=None):
        '''id of the product that the process has manipulated '''
        self.product_id = product_id
        '''Type of the product that the process has manipulated '''
        self.product_type = product_type
        '''creation date of the product that the process has manipulated '''
        self.product_date = product_date
        '''Informal description of the process'''
        self.job_desc = job_desc
        '''type of job execute by the process
           Can be initialize, download, pretest, configure, process, posttest,
           quicklook, validate, archive, clean,...'''
        self.proc_type = proc_type
        '''id of the parent of the process'''
        self.parent_job_id = parent_job_id
        '''id of the process'''
        self.job_id = job_id
        '''type of process executed by the process
           Can be a Yarn/Spark application ID (application_1234567),
           or a workflow job id (Airflow/Oozie/...)
        '''
        self.job_type = job_type
        '''start time of the execution of the process'''
        self.start_time = start_time
        '''end time of the execution of the process'''
        self.end_time = end_time
        '''duration of the process'''
        self.duration = None
        '''exit code of the process'''
        self.exit_code = exit_code
        '''message associated to the process'''
        self.message = message
        '''user under which the process was executed'''
        self.user = user
        '''hostname on which the process was executed'''
        self.host = host
        '''environment on which the process was executed'''
        self.environment = environment
        '''file manipulated/created/... by the process'''
        self.files = []
        '''Sparc configuration associated to the process'''
        self.configuration = None
        '''Unique id of this entry (populated if required)'''
        self.id = None

    def finalise(self):
        '''update the current process log information'''
        self.end_time = datetime.datetime.now()
        self.duration = (
            self.end_time - self.start_time).total_seconds()
        return self
